@@ -101,7 +101,7 @@ module Sass
         _to_gamut(GamutMapMethod.from_name(method, 'method'))
       end
 
-      # @return [Array<Numeric>]
+      # @return [Array<Numeric, nil>]
       def channels_or_nil
         [channel0_or_nil, channel1_or_nil, channel2_or_nil].freeze
       end
@@ -254,12 +254,12 @@ module Sass
         changed_color._to_space(_space)
       end
 
-      # return [Numeric]
+      # @return [Numeric]
       def alpha
         @alpha_or_nil.nil? ? 0 : @alpha_or_nil
       end
 
-      # return [::Boolean]
+      # @return [::Boolean]
       def legacy?
         _space.legacy?
       end
@@ -339,11 +339,11 @@ module Sass
       # @return [Integer]
       def hash
         @hash ||= [
-          _space,
-          channel0_or_nil&.finite? ? (channel0_or_nil * FuzzyMath::INVERSE_EPSILON).round : channel0_or_nil, # rubocop:disable Security/CompoundHash
-          channel1_or_nil&.finite? ? (channel1_or_nil * FuzzyMath::INVERSE_EPSILON).round : channel1_or_nil, # rubocop:disable Security/CompoundHash
-          channel2_or_nil&.finite? ? (channel2_or_nil * FuzzyMath::INVERSE_EPSILON).round : channel2_or_nil, # rubocop:disable Security/CompoundHash
-          alpha_or_nil&.finite? ? (alpha_or_nil * FuzzyMath::INVERSE_EPSILON).round : alpha_or_nil # rubocop:disable Security/CompoundHash
+          _space.name,
+          FuzzyMath._hash(channel0_or_nil),
+          FuzzyMath._hash(channel1_or_nil),
+          FuzzyMath._hash(channel2_or_nil),
+          FuzzyMath._hash(alpha_or_nil)
         ].hash
       end
 
@@ -569,8 +569,8 @@ module Sass
 
         return false if original.equal?(output)
 
-        output_channel = output.space.channels[output_channel_index]
-        original_channel = original.space.channels.find do |channel|
+        output_channel = output._space.channels[output_channel_index]
+        original_channel = original._space.channels.find do |channel|
           output_channel.analogous?(channel)
         end
 
